/*---------------------------------------------------------------------------*/
/*  Test skeleton for a "DIGIMODE-DLL"   to be loaded into Spectrum Lab.     */
/*     DO NOT SPECIALIZE THIS FILE, BUT MAKE A COPY AND RENAME !             */
/*---------------------------------------------------------------------------*/

// Creating DLLs without Borland's VCL-stuff :
//  Use the "Console Wizard" ,   but  **NOT**  "File..New..DLL" !
//  See Borlands Help System (yucc...) on:
//    - "Creating DLLs in C++Builder".
//        To create a non-VCL DLL, use the Console Wizard (!!!)
//    -  __declspec(dllexport) modifier (see header file !)
//    - "Example of creating a DLL"

/* Because Spectrum Lab will load the generated DLL 'step-by-step'
   with "LoadLibrary" and "GetProcAddress",  take care to:

   - Use precisely the names required for all routines in the DLL which will
     be called by Spectrum Lab. Disable the C++ "name mangling"
     by using 'extern "C"' in the function headers. Care for upper/lower case.
   - Use the standard parameter passing format,
     at least for the 'interface routines'. Use the modifier _stdcall for
     this, making the DLL compatible to DELPHI where 'stdcall' is avaible too.
     ('stdcall' is definitely NOT the "C"-standard, because the caller places
      parameters on the stack, and the called procedure has to REMOVE the
      argument from the stack... making flexible parameter lists impossible).

 */

#include <windows.h>
#include <stdio.h>    // no 'standard I/O' here, just sprintf() !
#pragma hdrstop
#include <condefs.h>

// The following C macro must be defined ONLY WHEN COMPILING THE DLL.
// The 'user' of the DLL uses the same header with the macro undefined.
#define __BUILDING_THE_DLL
#include "DIGMdlls.h"   // DLL "import"/"export" header for all Digimode-DLLs


//---------------------------------------------------------------------------
//   Important note about DLL memory management in a VCL DLL:
//
//
// If your DLL uses VCL and exports any functions that pass VCL String objects
// (or structs/classes containing nested Strings) as parameter or function
// results, you will need to build both your DLL project and any EXE projects
// that use your DLL with the dynamic RTL (the RTL DLL).  This will change your
// DLL and its calling EXE's to use BORLNDMM.DLL as their memory manager. In
// these cases, the file BORLNDMM.DLL should be deployed along with your DLL
// and the RTL DLL (CP3240MT.DLL). To avoid the requiring BORLNDMM.DLL in
// these situations, pass string information using "char *" or ShortString
// parameters and then link with the static RTL.
//
//---------------------------------------------------------------------------
#pragma argsused
int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void*)
{
  return 1;
}


/*-------- Implementation of all EXPORTED routines of this DLL -------------*/

//--------------------------------------------------------------------------
extern "C" char*  __EXPORT_TYPE _stdcall DIGM_GetParamString(long param_nr)
  // Exported function which returns STRINGS.
  // Input parameter:
  //   param_nr  defines which string shall be returned.
  //             Not implemented parameters may return NULL/nil
  //             or a dummy string. See case-list below.
  //  Return value:
  //    pointer to a null-terminated string (a 'PChar' in Delphi).
  //

{
  static char ret[256];
  switch(param_nr)
   {
     case 0: // DLL information string #1 : what's the mode & version ?
             sprintf(ret,"Digimode-Demo-DLL V1.0 in C++");
             break;
     case 1: // DLL information string #2 : copyright and author
             sprintf(ret,"by Wolfgang Buescher, DL4YHF");
             break;
     case 2: // DLL compilation date / version string
             sprintf(ret,"%s", __DATE__ );
             break;
     default: // unknown parameter number
             sprintf(ret,"unknown parameter #%ld", param_nr);
             break;
   }

  return ret;

} // end DIGM_GetParamString(long param_nr)


//--------------------------------------------------------------------------
extern "C" char*  __EXPORT_TYPE _stdcall DIGM_SetMode(long param_nr)
  // Exported function to set the ns STRINGS.
  // Input parameter:
  //   param_nr  defines which string shall be returned.
  //             Not implemented parameters may return NULL/nil
  //             or a dummy string. See case-list below.
  //  Return value:
  //    pointer to a null-terminated string (a 'PChar' in Delphi).
  //

{
  static char ret[256];
  switch(param_nr)
   {
     case 0: // DLL information string #1 : what's the mode & version ?
             sprintf(ret,"Digimode-Demo-DLL V1.0 in C++");
             break;
     case 1: // DLL information string #2 : copyright and author
             sprintf(ret,"by Wolfgang Buescher, DL4YHF");
             break;
     case 2: // DLL compilation date / version string
             sprintf(ret,"%s", __DATE__ );
             break;
     default: // unknown parameter number
             sprintf(ret,"unknown parameter #%ld", param_nr);
             break;
   }

  return ret;

} // end DIGM_GetParamString(long param_nr)


/***************************************************************************/
void DigiMode_Decoder(double *input_chunk, int chunk_size)
  /* Called by the real-time sound thread after a chunk of input samples
   * has been received.
   * NOTE THAT THE SAMPLES MUST NOT MODIFIED ! (see caller in SoundThd.cpp)
   * EVERY MICROSECOND COUNTS !!!!
   */

